@@ -25,7 +25,7 @@ def run(command, **kw):
         output = check_output(command, **kw)
     except CalledProcessError:
         return []
-    return output.decode().splitlines()
+    return output.decode(errors='replace').splitlines()
 
 def escape(s):
     """Escape the characters special to locate(1) globbing."""
@@ -63,16 +63,19 @@ def status_mercurial(path, ignore_set, options):
 
 def status_git(path, ignore_set, options):
     """Return text lines describing the status of a Git repository."""
-    # Check current branch:
+    # Check whether current branch is dirty:
     lines = [ l for l in run(('git', 'status', '-s', '-b'), cwd=path)
-              if (options.untracked or not l.startswith('?'))
-                 and (not l.startswith('##')) or (' [ahead ' in l)]
-    if len(lines):
-        return lines # changes detected, no need to check other branches
+              if (options.untracked or not l.startswith('?')) and not l.startswith('##')]
 
-    # Check other branches:
-    lines = [ l for l in run(('git', 'branch', '-v'), cwd=path)
-              if (' [ahead ' in l)]
+    # Check all branches for unpushed commits:
+    lines += [ l for l in run(('git', 'branch', '-v'), cwd=path)
+               if (' [ahead ' in l)]
+
+    # Check for non-tracking branches:
+    if options.non_tracking:
+        lines += [ l for l in run(('git', 'for-each-ref', '--format=[%(refname:short)]%(upstream)',
+                                   'refs/heads'), cwd=path)
+                   if l.endswith(']')]
     return lines
 
 def status_subversion(path, ignore_set, options):
@@ -103,6 +106,13 @@ def scan(repos, options):
     """Given a repository list [(path, vcsname), ...], scan each of them."""
     ignore_set = set()
     for directory, dotdir in repos:
+        ignore_this = any(pat in directory for pat in options.ignore_patterns)
+        if ignore_this:
+            if options.verbose:
+                print('Ignoring repo: {}'.format(directory))
+                print('')
+            continue
+
         vcsname, get_status = SYSTEMS[dotdir]
         lines = get_status(directory, ignore_set, options)
         if lines is None:  # signal that we should ignore this one
@@ -123,8 +133,13 @@ def main():
         help='manually walk file tree to find repositories (the default)')
     parser.add_option('-L', dest='follow_symlinks', action='store_true',
         help='follow symbolic links when walking file tree')
+    parser.add_option('-n', '--non-tracking', action='store_true',
+        help='print non-tracking branches (git only)')
     parser.add_option('-u', '--untracked', action='store_true',
         help='print untracked files (git only)')
+    parser.add_option('-I', dest='ignore_patterns', action='append',
+        default=[],
+        help='ignore any directory paths that contain the specified string')
     (options, args) = parser.parse_args()
 
     if not args:
