@@ -69,7 +69,7 @@ def checkouts(git_identity, tempdir, cc):
     os.mkdir(checkouts_dir)
 
     for system in 'git', 'hg', 'svn':
-        for state in 'clean', 'dirty':
+        for state in 'clean', 'dirty', 'ignore':
             d = os.path.join(checkouts_dir, system + '-' + state)
 
             # Create the repo:
@@ -97,7 +97,7 @@ def checkouts(git_identity, tempdir, cc):
                 cc([system, 'commit', '-m', 'Add more maxim'], cwd=d)
 
             # Make the master branch dirty:
-            if state == 'dirty':
+            if state == 'dirty' or state == 'ignore':
                 with open(file_to_edit, 'a') as f:
                     f.write(even_more_maxim)
 
@@ -170,45 +170,108 @@ def test_uncommitted(checkouts):
         {path}/git-dirty - Git
          M {filename}
 
+        {path}/git-ignore - Git
+         M {filename}
+
         {path}/hg-dirty - Mercurial
          M {filename}
 
+        {path}/hg-ignore - Mercurial
+         M {filename}
+
         {path}/svn-dirty - Subversion
          M       {filename}
 
+        {path}/svn-ignore - Subversion
+         M       {filename}
+
         """).format(path=checkouts, filename=filename)
 
     assert actual_output == expected_output
 
-def test_unpushed_current_branch(clones):
-    """Do we detect when the current branch has unpushed changes?"""
-    actual_output = run(clones)
+def test_uncommitted_ignore_one(checkouts):
+    """Does -I correctly ignore directories?"""
+    actual_output = run('-I', 't-ignore', checkouts)
 
-    # Only the clone for which the current branch is ahead:
+    # All dirty checkouts and only them:
     expected_output = dedent("""\
-        {path}/git-complex - Git
-        ## not-behind-ahead...origin/master [ahead 1]
+        {path}/git-dirty - Git
+         M {filename}
 
-        """).format(path=clones)
+        {path}/hg-dirty - Mercurial
+         M {filename}
+
+        {path}/hg-ignore - Mercurial
+         M {filename}
+
+        {path}/svn-dirty - Subversion
+         M       {filename}
+
+        {path}/svn-ignore - Subversion
+         M       {filename}
+
+        """).format(path=checkouts, filename=filename)
 
     assert actual_output == expected_output
 
-def test_unpushed_other_branches(clones, cc):
-    """Do we detect a non-checked-out with unpushed changes?"""
-    complex_clone_dir = os.path.join(clones, 'git-complex')
-    try:
-        # Especially for this test, check out a branch which is up-to-date:
-        cc(['git', 'checkout', 'master'], cwd=complex_clone_dir)
-        actual_output = run(clones)
-    finally:
-        cc(['git', 'checkout', 'not-behind-ahead'], cwd=complex_clone_dir)
+def test_uncommitted_ignore_two_verbose(checkouts):
+    """Can -I be offered twice?"""
+    actual_output = run('-I', 't-ignore', '-I', 'g-ignore', '-v', checkouts)
 
-    # All ahead branches and only them:
+    # All dirty checkouts and only them:
+    expected_output = dedent("""\
+        {path}/git-clean - Git
+
+        {path}/git-dirty - Git
+         M {filename}
+
+        Ignoring repo: {path}/git-ignore
+
+        {path}/hg-clean - Mercurial
+
+        {path}/hg-dirty - Mercurial
+         M {filename}
+
+        Ignoring repo: {path}/hg-ignore
+
+        {path}/svn-clean - Subversion
+
+        {path}/svn-dirty - Subversion
+         M       {filename}
+
+        {path}/svn-ignore - Subversion
+         M       {filename}
+
+        """).format(path=checkouts, filename=filename)
+
+    assert actual_output == expected_output
+
+def test_unpushed(clones):
+    """Do we detect when any branch (checked out or not) has unpushed
+    changes?"""
+    actual_output = run(clones)
+
+    # All ahead branches and only them (the checked-out branch is marked with a
+    # star):
     expected_output_regex = re.compile(dedent("""\
         ^{path}/git-complex - Git
           behind-ahead         .* \[ahead 1, behind 1\] Even more maxim
-          not-behind-ahead     .* \[ahead 1\] Even more maxim
+        \* not-behind-ahead     .* \[ahead 1\] Even more maxim
 
         $""").format(path=clones))
 
     assert expected_output_regex.match(actual_output) is not None
+
+def test_non_tracking(checkouts):
+    """Do we detect non-tracking branches?"""
+    clean_git_repo = os.path.join(checkouts, 'git-clean')
+    actual_output = run(clean_git_repo, '-n')
+
+    # The 'master' branch isn't tracking any remote:
+    expected_output = dedent("""\
+        {path} - Git
+        [master]
+
+        """).format(path=clean_git_repo)
+
+    assert actual_output == expected_output
